//+------------------------------------------------------------------+
//|                                  Expert main module - Strategy 2 |
//|                               Copyright (c) 2012-2014 Roman Rich |
//|                       Trades many symbols following one strategy |
//+------------------------------------------------------------------+
#property            version                 "1.01"
#property            link                    "https://www.facebook.com/roman.i.rich"
#property            copyright               "Copyright (c) 2012-2014 Roman Rich"
#property            description             "Strategy 2:"
#property            description             "  Break through the support/resistance line"
#property            description             "  SL - reverse"
#property            description             "  TP - no"
#property            description             "  Trail - by support/resistance line"
//--- List of symbols
#include             "symbol_list_2.mqh"
//---
int                  bars_count;             // The number of bars on a chart                                           
double               spread;                 // Spread
//---
int                  count_1;                // Counter
int                  count_2;                // Counter
int                  count_3;                // Counter
double               temp_1;                 // Temporary variable
int                  temp_2;                 // Temporary variable
//---
datetime             date_time[];            // An array for storing the open time of the timeseries bars
double               price_open[];           // An array for storing the open prices of the timeseries bars
double               price_high[];           // An array for storing the High prices of the timeseries bars
double               price_low[];           // An array for storing the Low prices of the timeseries bars
double               price_close[];          // An array for storing the Close prices of the timeseries bars
ulong                volumes[];              // An array for storing the volumes of the timeseries bars
//---
uint                 symbol_digit=4;         // The number of symbols for chart drawing
uint                 digit_2_orders;         // The number of symbols for order placing
int                  symbols_total;          // Total symbols in MarketWatch
int                  market_watch_symbol;    // Current symbol in MarketWatch
//---
datetime             prev_bar_time_dig[1];   // Star time of the last but one bar in a numeric format
datetime             cur_bar_time_dig[1];    // Star time of the last bar in a numeric format
//---
MqlDateTime          prev_bar_time_struct;   // Star time of the last but one bar in a structure format
MqlDateTime          cur_bar_time_struct;    // Star time of the last bar in a structure format
MqlTradeRequest      trade_request;          // Trade request
MqlTradeCheckResult  trade_check;            // Checking the trade request 
MqlTradeResult       trade_result;           // Result of a trade request
MqlTick              last_tick;              // Current prices
//---
string               company;                // A supplier of quotes
string               account;                // Account type
string               file_name;              // File name
int                  handle;                 // A pointer to the file
string               symbol;                 // Symbol name
double               calc_margin;            // Margin
double               order_vol;              // Claimed deal volume
ulong                ticket=0;               // The ticket of the order/deal
//+------------------------------------------------------------------+
//| The structure of the XO chart                                    |
//+------------------------------------------------------------------+
struct PNF_CHART
  {
   uint              column_num;             // Column number
   datetime          last_date_time;         // Date/time of the last value in the column
   uchar             column_type;            // Column type:  or 
   double            min_column_price;       // Minimum value on the column
   double            max_column_price;       // Maximum value in the column
   ulong             column_volume;          // Volume in the column
   double            supp_price;             // Support price in the column
   double            resist_price;           // Resistance price in the column
  };
//--- The point and figure chart
PNF_CHART pnf[];
//---
int                  column_count;           // The index of the current column
double               double_box;             // Box size in pips
double               start_min;              // The minimum of the earliest bar cast to the box size
double               start_max;              // The maximum of the earliest bar cast to the box size
double               cur_min;                // The minimum of the current bar cast to the box size
double               cur_max;                // The maximum of the current bar cast to the box size
double               rev_min;                // The minimum required for column reversal to O cast to the box size
double               rev_max;                // The maximum required for column reversal to X cast to the box size
double               cont_min;               // The minimum required for the continuation of column O, cast to the box size
double               cont_max;               // The maximum required for the continuation of column X, cast to the box size
ulong                cur_vol;                // The volume of the current bar
uchar                cur_column;             // Column type: X or O
uchar                trend;                  // Current trend type: U or D
double               temp_minimum_price[];   // Temporary array of minimums of all columns of the XO chart
double               temp_maximum_price[];   // Temporary array of maximums of all columns of the XO chart
int                  order_col_number=0;     // The index of the column at which an order is placed
//+------------------------------------------------------------------+
//| Enumeration of possible box sizes                                |
//+------------------------------------------------------------------+
enum  BOX_SIZE
  {
   b_1=1,
   b_2=2,
   b_3=3,
   b_4=4,
   b_5=5,
   b_10=10,
   b_20=20,
   b_50=50,
   b_100=100,
   b_200=200,
   b_500=500
  };
//+------------------------------------------------------------------+
//| Enumeration of possible reversals                                |
//+------------------------------------------------------------------+
enum  REVERSE_SIZE
  {
   one=1,
   two,
   three,
   four,
   five
  };
//+------------------------------------------------------------------+
//| Input parameters                                                |
//+------------------------------------------------------------------+
sinput string          delimiter_1="";                // ===MONEY MANAGEMENT VARIABLES===
input  bool            is_const_lot=true;             // Trade minimum lot?
input  double          risk=0.10;                     // Max Drawdown, %
input  double          opt_f=0.01;                    // Risk per trade, %
sinput string          delimiter_2="";                // ===COMMON VARIABLES===
input  datetime        begin_of_test=D'2012.01.01';   // Testing Starts
input  SYMBOLS         selected_symbol;               // Symbol
input  ENUM_TIMEFRAMES time_frame=PERIOD_M1;          // Timeframe
input  ulong           magic=666;                     // Magic of the EA
sinput string          delimiter_3="";                // ===CHART VARIABLES===
input  BOX_SIZE        box=b_100;                     // Box size in pips (2 or 4 bit quotes)
input  REVERSE_SIZE    reverse=three;                 // Reversal, boxes
sinput string          delimiter_4="";                // ===STRATEGY VARIABLES===
//+------------------------------------------------------------------+
//| Typical OnInit() event handler                                   |
//|     Set date/time                                                |
//|     Select symbol                                                |
//|     Select number of digits in quotes                            |
//|     Primary calculation of the chart                             |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Initialize environment variables
   prev_bar_time_dig[0]=D'2001.01.01';
   TimeToStruct(prev_bar_time_dig[0],prev_bar_time_struct);
//--- Selecting symbol
   if((MQLInfoInteger(MQL_TESTER)==true) || (MQLInfoInteger(MQL_OPTIMIZATION)==true))
      SelectSymbol();
   else
      symbol=Symbol();
//--- Set the number of digits in quotes
   if(SymbolInfoInteger(symbol,SYMBOL_DIGITS)<=3)
      digit_2_orders=2;
   else
      digit_2_orders=4;
//--- Initial chart drawing
   CreatePNFArray();
//--- Complete initialization
   return(INIT_SUCCEEDED);
  };
//+------------------------------------------------------------------+
//| Typical tick handler OnTick()                                    |
//|     Draw chart only on complete bars, and first                  |
//|     check if it's a new bar                                      |
//|     If the bar is new and there is a position, check             |
//|     if you need to move stop loss;                               |
//|     when there is no position, check                             |
//|     if there is a condition to open a deal                       |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- If it's a new bar
   if(IsNewBar()==true)
     {
      RecalcIndicators();
      //--- tester/optimizer mode?
      if((MQLInfoInteger(MQL_TESTER)==true) || (MQLInfoInteger(MQL_OPTIMIZATION)==true))
        {
         //--- Is it the testing period?
         if(cur_bar_time_dig[0]>begin_of_test)
           {
            //--- If there is an open position on the symbol
            if(PositionSelect(symbol)==true)
               //--- check if we need to move SL; if need, move it
               TrailCondition();
            //--- If there are no positions
            else
            //--- check if we need to open a position; if we need, open it
               TradeCondition();
           }
        }
      else
        {
         //--- If there is an open position on the symbol
         if(PositionSelect(symbol)==true)
            //--- check if we need to move SL; if need, move it
            TrailCondition();
         //--- If there are no positions
         else
         //--- check if we need to open a position; if we need, open it
            TradeCondition();
        }

     };
  };
//+------------------------------------------------------------------+
//| The results of the strategy in the testing mode                  |
//+------------------------------------------------------------------+
double OnTester()
  {
   if(TesterStatistics(STAT_EQUITY_DDREL_PERCENT)>(risk*100))
      return(0);
   else
      return(NormalizeDouble(TesterStatistics(STAT_PROFIT),(uint)SymbolInfoInteger(symbol,SYMBOL_DIGITS)));
  };
//+------------------------------------------------------------------+
//| Function checks trade conditions for opening a deal              |
//+------------------------------------------------------------------+
void TradeCondition()
  {
   if(order_col_number!=column_count)
      //--- Are there any orders on the symbol?
     {
      if(OrdersTotal()>0)
        {
         //--- Delete them!
         for(int loc_count_1=0;loc_count_1<OrdersTotal();loc_count_1++)
           {
            ticket=OrderGetTicket(loc_count_1);
            if(!OrderSelect(ticket)) Print("Failed to select order #",ticket);
            if(OrderGetString(ORDER_SYMBOL)==symbol)
              {
               trade_request.order=ticket;
               trade_request.action=TRADE_ACTION_REMOVE;
               if(!OrderSend(trade_request,trade_result)) Print("Failed to send order #",trade_request.order);
              };
           };
         order_col_number=column_count;
         return;
        }
      else
        {
         order_col_number=column_count;
         return;
        }
     }
   else
   if(MathPow(10,pnf[column_count-1].resist_price)>SymbolInfoDouble(symbol,SYMBOL_ASK))
     {
      //--- Conditions for BUY met; let's see if there are any pending Buy orders for the symbol with the price we need?
      trade_request.price=NormalizeDouble(MathPow(10,pnf[column_count-1].resist_price),digit_2_orders);
      trade_request.sl=NormalizeDouble(MathPow(10,pnf[column_count-1].resist_price-(reverse-1)*double_box),digit_2_orders);
      trade_request.type=ORDER_TYPE_BUY_STOP;
      if(OrderSelect(ticket)==false)
         //--- No pending orders - place an order
        {
         PlaceOrder();
         order_col_number=column_count;
        }
      else
      //--- If there is a pending order
        {
         //--- what is the type and price of the pending order?
         if((OrderGetInteger(ORDER_TYPE)==ORDER_TYPE_SELL_STOP) || 
            ((OrderGetInteger(ORDER_TYPE)==ORDER_TYPE_BUY_STOP) && (OrderGetDouble(ORDER_PRICE_OPEN)!=trade_request.price)))
           {
            //--- The wrong type or the price differs - close the order
            trade_request.order=ticket;
            trade_request.action=TRADE_ACTION_REMOVE;
            if(!OrderSend(trade_request,trade_result)) Print("Failed to send order #",trade_request.order);
            //--- open with the right price
            PlaceOrder();
            order_col_number=column_count;
           };
        };
      return;
     }
   else
   if(MathPow(10,pnf[column_count-1].resist_price)<SymbolInfoDouble(symbol,SYMBOL_ASK))
     {
      //--- Conditions for SELL met; let's see if there are any pending Sell orders for the symbol with the price we need?
      trade_request.price=NormalizeDouble(MathPow(10,pnf[column_count-1].supp_price),digit_2_orders);
      trade_request.sl=NormalizeDouble(MathPow(10,pnf[column_count-1].supp_price+(reverse-1)*double_box),digit_2_orders);
      trade_request.type=ORDER_TYPE_SELL_STOP;
      if(OrderSelect(ticket)==false)
         //--- No pending orders - place an order
        {
         PlaceOrder();
         order_col_number=column_count;
        }
      else
      //--- If there is a pending order
        {
         //--- what is the type and price of the pending order?
         if((OrderGetInteger(ORDER_TYPE)==ORDER_TYPE_BUY_STOP) || 
            ((OrderGetInteger(ORDER_TYPE)==ORDER_TYPE_SELL_STOP) && (OrderGetDouble(ORDER_PRICE_OPEN)!=trade_request.price)))
           {
            //--- The wrong type or the price differs - close the order
            trade_request.order=ticket;
            trade_request.action=TRADE_ACTION_REMOVE;
            if(!OrderSend(trade_request,trade_result)) Print("Failed to send order #",trade_request.order);
            //--- open with the right price
            PlaceOrder();
            order_col_number=column_count;
           };
        };
      return;
     }
   else
      return;
  };
//+------------------------------------------------------------------+
//| The function checks conditions for moving Stop Loss              |
//+------------------------------------------------------------------+
void TrailCondition()
  {
   if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY)
      trade_request.sl=NormalizeDouble(MathMax(SymbolInfoDouble(symbol,SYMBOL_ASK),MathPow(10,pnf[column_count-1].max_column_price-reverse*double_box)),digit_2_orders);
   else
      trade_request.sl=NormalizeDouble(MathMin(SymbolInfoDouble(symbol,SYMBOL_BID),MathPow(10,pnf[column_count-1].min_column_price+reverse*double_box)),digit_2_orders);
   if(PositionGetDouble(POSITION_SL)!=trade_request.sl)
      PlaceTrailOrder();
  };
//+------------------------------------------------------------------+
//| The function places an order with a precalculated volume         |
//+------------------------------------------------------------------+
void PlaceOrder()
  {
//--- Variables for lot calculation
   uint digits_2_lot=(uint)SymbolInfoInteger(symbol,SYMBOL_DIGITS);
   double trade_risk=AccountInfoDouble(ACCOUNT_EQUITY)*opt_f;
   double one_tick_loss_min_lot=SymbolInfoDouble(symbol,SYMBOL_TRADE_TICK_VALUE_LOSS)*SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP);
//--- Fill in the main fields of the request
   trade_request.magic=magic;
   trade_request.symbol=symbol;
   trade_request.action=TRADE_ACTION_PENDING;
   trade_request.tp=NULL;
   trade_request.comment=NULL;
   trade_request.type_filling=NULL;
   trade_request.stoplimit=NULL;
   trade_request.type_time=NULL;
   trade_request.expiration=NULL;
   if(is_const_lot==true)
     {
      order_vol=SymbolInfoDouble(symbol,SYMBOL_VOLUME_MIN);
     }
   else
     {
      order_vol=trade_risk/(MathAbs(trade_request.price-trade_request.sl)*MathPow(10,digits_2_lot)*one_tick_loss_min_lot)*SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP);
      order_vol=MathMax(order_vol,SymbolInfoDouble(symbol,SYMBOL_VOLUME_MIN));
      if(SymbolInfoDouble(symbol,SYMBOL_VOLUME_LIMIT)!=0) order_vol=MathMin(order_vol,SymbolInfoDouble(symbol,SYMBOL_VOLUME_LIMIT));
      order_vol=NormalizeDouble(order_vol,(int)MathAbs(MathLog10(SymbolInfoDouble(symbol,SYMBOL_VOLUME_STEP))));
     }
//--- Place an order
   while(order_vol>0)
     {
      trade_request.volume=MathMin(order_vol,SymbolInfoDouble(symbol,SYMBOL_VOLUME_MAX));
      if(!OrderSend(trade_request,trade_result)) Print("Failed to send order #",trade_request.order);
      order_vol=order_vol-SymbolInfoDouble(symbol,SYMBOL_VOLUME_MAX);
     };
   ticket=trade_result.order;
  };
//+------------------------------------------------------------------+
//| The function moves the stop loss order                           |
//+------------------------------------------------------------------+
void PlaceTrailOrder()
  {
   trade_request.magic=magic;
   trade_request.symbol=symbol;
   trade_request.action=TRADE_ACTION_SLTP;
   if(!OrderSend(trade_request,trade_result)) Print("Failed to send order #",trade_request.order);
  };
//+------------------------------------------------------------------+
//| Preparing bar data and calling the function that draws the chart |
//+------------------------------------------------------------------+
void RecalcIndicators()
  {
//--- Fill in arrays and variables using the data of the last bar,...
   CopyTime(symbol,time_frame,1,1,date_time);
   CopyHigh(symbol,time_frame,1,1,price_high);
   CopyLow(symbol,time_frame,1,1,price_low);
   CopyRealVolume(symbol,time_frame,1,1,volumes);
//--- Logarithm
   price_low[0]=NormalizeDouble(MathLog10(price_low[0]),symbol_digit);
   price_high[0]=NormalizeDouble(MathLog10(price_high[0]),symbol_digit);
   cur_min=MathNorm(price_low[0],double_box,true,symbol_digit);
   cur_max=MathNorm(price_high[0],double_box,false,symbol_digit);
   cur_vol=volumes[0];
//--- ...continue drawing chart PNF,...
   UpdatePNF();
//--- ...if a new column has been added in the PNF chart, extend auxiliary arrays,...
   if(ArraySize(temp_minimum_price)!=column_count)
     {
      ArrayResize(temp_minimum_price,column_count,0);
      ArrayResize(temp_maximum_price,column_count,0);
      pnf[column_count-1].supp_price=0.0;
      pnf[column_count-1].resist_price=0.0;
     };
//--- ...update trend lines
   temp_minimum_price[column_count-1]=pnf[column_count-1].min_column_price;
   temp_maximum_price[column_count-1]=pnf[column_count-1].max_column_price;
   UpdateTrend();
  };
//+------------------------------------------------------------------+
//| The function of initial chart drawing                            |
//+------------------------------------------------------------------+
void CreatePNFArray()
  {
   column_count=1;
   bars_count=MathMin(100000,Bars(symbol,time_frame));
   cur_column='.';
   trend='.';
//--- Preparing the required arrays
   ArrayFree(pnf);
   ArrayFree(date_time);
   ArrayFree(price_high);
   ArrayFree(price_low);
   ArrayFree(volumes);
   ArrayFree(temp_minimum_price);
   ArrayFree(temp_maximum_price);
   ArrayResize(pnf,bars_count,0);
   ArrayResize(date_time,bars_count,0);
   ArrayResize(price_high,bars_count,0);
   ArrayResize(price_low,bars_count,0);
   ArrayResize(volumes,bars_count,0);
//--- Fill them in with quotes
   CopyTime(symbol,time_frame,0,bars_count,date_time);
   CopyHigh(symbol,time_frame,0,bars_count,price_high);
   CopyLow(symbol,time_frame,0,bars_count,price_low);
   CopyRealVolume(symbol,time_frame,0,bars_count,volumes);
//--- Logarithm
   for(count_1=0;count_1<bars_count;count_1++)
     {
      price_low[count_1]=NormalizeDouble(MathLog10(price_low[count_1]),symbol_digit);
      price_high[count_1]=NormalizeDouble(MathLog10(price_high[count_1]),symbol_digit);
     };
//--- Initialize intermediate variables
   double_box=NormalizeDouble((double)box/MathPow(10.0,(double)symbol_digit),symbol_digit);
   start_min=MathNorm(price_low[0],double_box,true,symbol_digit);
   start_max=MathNorm(price_high[0],double_box,false,symbol_digit);
   cont_min=NormalizeDouble(start_min-double_box,symbol_digit);
   cont_max=NormalizeDouble(start_max+double_box,symbol_digit);
   rev_min=NormalizeDouble(start_max-reverse*double_box,symbol_digit);
   rev_max=NormalizeDouble(start_min+reverse*double_box,symbol_digit);
   cur_vol=volumes[0];
//--- and some fields of the first chart entry
   pnf[column_count-1].column_num=column_count;
   pnf[column_count-1].column_type=cur_column;
   pnf[column_count-1].min_column_price=start_min;
   pnf[column_count-1].max_column_price=start_max;
   pnf[column_count-1].column_volume=cur_vol;
//--- Draw the point and figure chart
   for(count_1=1;count_1<bars_count;count_1++)
     {
      cur_min=MathNorm(price_low[count_1],double_box,true,symbol_digit);
      cur_max=MathNorm(price_high[count_1],double_box,false,symbol_digit);
      cur_vol=volumes[count_1];
      UpdatePNF();
     };
//--- Draw trend lines
//--- Prepare and initialize required arrays
   ArrayResize(temp_minimum_price,column_count,0);
   ArrayResize(temp_maximum_price,column_count,0);
   for(count_2=0;count_2<column_count;count_2++)
     {
      pnf[count_2].supp_price=0.0;
      pnf[count_2].resist_price=0.0;
      temp_minimum_price[count_2]=pnf[count_2].min_column_price;
      temp_maximum_price[count_2]=pnf[count_2].max_column_price;
     }
//--- and some fields of the first chart entry (initial trend has been defined earlier)
   if(trend=='U')
     {
      pnf[0].supp_price=NormalizeDouble(temp_minimum_price[0]-double_box,symbol_digit);
      pnf[0].resist_price=0.0;
     }
   else
     {
      pnf[0].supp_price=0.0;
      pnf[0].resist_price=NormalizeDouble(temp_maximum_price[0]+double_box,symbol_digit);
     }
   count_2=count_3=1;
   while(count_2<column_count)
      CalcTrend();
//--- Before the tick-by-tick operation, shorten the size of arrays to 1
   ArrayFree(date_time);
   ArrayFree(price_high);
   ArrayFree(price_low);
   ArrayFree(volumes);
   ArrayResize(date_time,1,0);
   ArrayResize(price_high,1,0);
   ArrayResize(price_low,1,0);
   ArrayResize(volumes,1,0);
  };
//+------------------------------------------------------------------+
//| The function continues chart drawing on each new bar             |
//+------------------------------------------------------------------+
void UpdatePNF()
  {
   switch(cur_column)
     {
      //--- On the first bars it is yet unclear what the first column will be : X or O
      case '.':
        {
         //--- In these conditions, the first column is X, and the trend is voluntarily is set as Down
         if(cur_max>=rev_max)
           {
            cur_column='X';
            cont_max=NormalizeDouble(cur_max+double_box,symbol_digit);
            rev_min=NormalizeDouble(cur_max-reverse*double_box,symbol_digit);
            trend='D';
            pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
            pnf[column_count-1].column_type=cur_column;
            pnf[column_count-1].max_column_price=cur_max;
            break;
           };
         //--- In these conditions, the first column is O, and the trend is voluntarily is set as Up
         if(cur_min<=rev_min)
           {
            cur_column='O';
            cont_min=NormalizeDouble(cur_min-double_box,symbol_digit);
            rev_max=NormalizeDouble(cur_min+reverse*double_box,symbol_digit);
            trend='U';
            pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
            pnf[column_count-1].column_type=cur_column;
            pnf[column_count-1].min_column_price=cur_min;
            break;
           };
         break;
        };
      //--- It is already clear that the column is X
      case 'X':
        {
         //--- and it continues under this condition
         if(cur_max>=cont_max)
           {
            cont_max=NormalizeDouble(cur_max+double_box,symbol_digit);
            rev_min=NormalizeDouble(cur_max-reverse*double_box,symbol_digit);
            pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
            pnf[column_count-1].max_column_price=cur_max;
            break;
           };
         //--- and reverses to O under this condition
         if(cur_min<=rev_min)
           {
            cur_column='O';
            column_count++;
            cont_min=NormalizeDouble(cur_min-double_box,symbol_digit);
            rev_max=NormalizeDouble(cur_min+reverse*double_box,symbol_digit);
            pnf[column_count-1].column_num=column_count;
            pnf[column_count-1].column_volume=cur_vol;
            pnf[column_count-1].column_type=cur_column;
            pnf[column_count-1].min_column_price=cur_min;
            pnf[column_count-1].max_column_price=NormalizeDouble(pnf[column_count-2].max_column_price-double_box,symbol_digit);
            break;
           };
         break;
        };
      //--- It is clear that the column is O
      case 'O':
        {
         //--- and it continues under this condition
         if(cur_min<=cont_min)
           {
            cont_min=NormalizeDouble(cur_min-double_box,symbol_digit);
            rev_max=NormalizeDouble(cur_min+reverse*double_box,symbol_digit);
            pnf[column_count-1].column_volume=pnf[column_count-1].column_volume+cur_vol;
            pnf[column_count-1].min_column_price=cur_min;
            break;
           };
         //--- and reverses to X under this condition
         if(cur_max>=rev_max)
           {
            cur_column='X';
            column_count++;
            cont_max=NormalizeDouble(cur_max+double_box,symbol_digit);
            rev_min=NormalizeDouble(cur_max-reverse*double_box,symbol_digit);
            pnf[column_count-1].column_num=column_count;
            pnf[column_count-1].column_volume=cur_vol;
            pnf[column_count-1].column_type=cur_column;
            pnf[column_count-1].min_column_price=NormalizeDouble(pnf[column_count-2].min_column_price+double_box,symbol_digit);
            pnf[column_count-1].max_column_price=cur_max;
            break;
           };
         break;
        };
     };
  };
//+------------------------------------------------------------------+
//| The function of initial trend detection                          |
//+------------------------------------------------------------------+
void CalcTrend()
  {
   switch(trend)
     {
      //--- Current trend - Up
      case 'U':
        {
         temp_1=NormalizeDouble(pnf[count_2-1].supp_price+double_box,symbol_digit);
         //--- if the condition is true, it continues
         if(temp_1<temp_minimum_price[count_2])
           {
            pnf[count_2].supp_price=temp_1;
            count_2++;
            break;
           }
         //--- if false, it changes to Down
         else
           {
            pnf[count_2].supp_price=-10.0;
            trend='D';
            count_3=ArrayMaximum(temp_maximum_price,count_3,count_2-count_3);
            temp_2=count_3;
            temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
            while(count_3<count_2)
              {
               if(temp_1>temp_maximum_price[count_3])
                 {
                  count_3++;
                  temp_1=NormalizeDouble(temp_1-double_box,symbol_digit);
                 }
               else
                 {
                  count_3=ArrayMaximum(temp_maximum_price,count_3,count_2-count_3);
                  temp_2=count_3;
                  temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
                 };
              };
            count_3=temp_2;
            temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
            pnf[count_3].resist_price=temp_1;
            count_3++;
            while(count_3<=count_2)
              {
               temp_1=NormalizeDouble(pnf[count_3-1].resist_price-double_box,symbol_digit);
               pnf[count_3].resist_price=temp_1;
               count_3++;
              }
            count_3=temp_2;
            count_2++;
            break;
           };
        };
      //--- Current trend - Down
      case 'D':
        {
         temp_1=NormalizeDouble(pnf[count_2-1].resist_price-double_box,symbol_digit);
         //--- if the condition is true, it continues
         if(temp_1>temp_maximum_price[count_2])
           {
            pnf[count_2].resist_price=temp_1;
            count_2++;
            break;
           }
         //--- if false, it changes to Up
         else
           {
            pnf[count_2].resist_price=-10.0;
            trend='U';
            count_3=ArrayMinimum(temp_minimum_price,count_3,count_2-count_3);
            temp_2=count_3;
            temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
            while(count_3<count_2)
              {
               if(temp_1<temp_minimum_price[count_3])
                 {
                  count_3++;
                  temp_1=NormalizeDouble(temp_1+double_box,symbol_digit);
                 }
               else
                 {
                  count_3=ArrayMinimum(temp_minimum_price,count_3,count_2-count_3);
                  temp_2=count_3;
                  temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
                 };
              };
            count_3=temp_2;
            temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
            pnf[count_3].supp_price=temp_1;
            count_3++;
            while(count_3<=count_2)
              {
               temp_1=NormalizeDouble(pnf[count_3-1].supp_price+double_box,symbol_digit);
               pnf[count_3].supp_price=temp_1;
               count_3++;
              }
            count_3=temp_2;
            count_2++;
            break;
           };
        };
     };
  };
//+------------------------------------------------------------------+
//| The function continues to draw trend in each new bar             |
//+------------------------------------------------------------------+
void UpdateTrend()
  {
   switch(trend)
     {
      //--- Current trend - Up
      case 'U':
        {
         temp_1=NormalizeDouble(pnf[column_count-2].supp_price+double_box,symbol_digit);
         //--- if the condition is true, it continues
         if(temp_1<temp_minimum_price[column_count-1])
           {
            pnf[column_count-1].supp_price=temp_1;
            break;
           }
         //--- if false, it changes to Down
         else
           {
            pnf[column_count-1].supp_price=-10.0;
            trend='D';
            count_3=ArrayMaximum(temp_maximum_price,count_3,column_count-1-count_3);
            temp_2=count_3;
            temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
            while(count_3<column_count-1)
              {
               if(temp_1>temp_maximum_price[count_3])
                 {
                  count_3++;
                  temp_1=NormalizeDouble(temp_1-double_box,symbol_digit);
                 }
               else
                 {
                  count_3=ArrayMaximum(temp_maximum_price,count_3,column_count-1-count_3);
                  temp_2=count_3;
                  temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
                 };
              };
            count_3=temp_2;
            temp_1=NormalizeDouble(temp_maximum_price[count_3]+double_box,symbol_digit);
            pnf[count_3].resist_price=temp_1;
            count_3++;
            while(count_3<=column_count-1)
              {
               temp_1=NormalizeDouble(pnf[count_3-1].resist_price-double_box,symbol_digit);
               pnf[count_3].resist_price=temp_1;
               count_3++;
              }
            count_3=temp_2;
            break;
           };
        };
      //--- Current trend - Down
      case 'D':
        {
         temp_1=NormalizeDouble(pnf[column_count-2].resist_price-double_box,symbol_digit);
         //--- if the condition is true, it continues
         if(temp_1>temp_maximum_price[column_count-1])
           {
            pnf[column_count-1].resist_price=temp_1;
            break;
           }
         //--- if false, it changes to Up
         else
           {
            pnf[column_count-1].resist_price=-10.0;
            trend='U';
            count_3=ArrayMinimum(temp_minimum_price,count_3,column_count-1-count_3);
            temp_2=count_3;
            temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
            while(count_3<column_count-1)
              {
               if(temp_1<temp_minimum_price[count_3])
                 {
                  count_3++;
                  temp_1=NormalizeDouble(temp_1+double_box,symbol_digit);
                 }
               else
                 {
                  count_3=ArrayMinimum(temp_minimum_price,count_3,column_count-1-count_3);
                  temp_2=count_3;
                  temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
                 };
              };
            count_3=temp_2;
            temp_1=NormalizeDouble(temp_minimum_price[count_3]-double_box,symbol_digit);
            pnf[count_3].supp_price=temp_1;
            count_3++;
            while(count_3<=column_count-1)
              {
               temp_1=NormalizeDouble(pnf[count_3-1].supp_price+double_box,symbol_digit);
               pnf[count_3].supp_price=temp_1;
               count_3++;
              }
            count_3=temp_2;
            break;
           };
        };
     };
  };
//+------------------------------------------------------------------+
//| Adjusting the price to the box size:                             |
//|     delete any double-number by the box size,                    |
//|     if true - rounding up, false - rounding down,                |
//|     set the accuracy equal to symbol's number of digits          |
//+------------------------------------------------------------------+
double MathNorm(double value,double var_box,bool vect,uint prec)
  {
   if(vect==true)
      return(NormalizeDouble(MathCeil(value/var_box)*var_box,prec));
   else
      return(NormalizeDouble(MathFloor(value/var_box)*var_box,prec));
  };
//+------------------------------------------------------------------+
//| Define whether a new bar has opened                              |
//+------------------------------------------------------------------+
bool IsNewBar()
  {
   CopyTime(symbol,time_frame,0,1,cur_bar_time_dig);
   TimeToStruct(cur_bar_time_dig[0],cur_bar_time_struct);
   if(time_frame==PERIOD_M1||
      time_frame==PERIOD_M2||
      time_frame==PERIOD_M3||
      time_frame==PERIOD_M4||
      time_frame==PERIOD_M5||
      time_frame==PERIOD_M6||
      time_frame==PERIOD_M10||
      time_frame==PERIOD_M12||
      time_frame==PERIOD_M15||
      time_frame==PERIOD_M20||
      time_frame==PERIOD_M30)
      if(cur_bar_time_struct.min!=prev_bar_time_struct.min)
        {
         prev_bar_time_dig[0]=cur_bar_time_dig[0];
         TimeToStruct(prev_bar_time_dig[0],prev_bar_time_struct);
         return(true);
        };
   if(time_frame==PERIOD_H1||
      time_frame==PERIOD_H2||
      time_frame==PERIOD_H3||
      time_frame==PERIOD_H4||
      time_frame==PERIOD_H6||
      time_frame==PERIOD_H8||
      time_frame==PERIOD_H12)
      if(cur_bar_time_struct.hour!=prev_bar_time_struct.hour)
        {
         prev_bar_time_dig[0]=cur_bar_time_dig[0];
         TimeToStruct(prev_bar_time_dig[0],prev_bar_time_struct);
         return(true);
        };
   if(time_frame==PERIOD_D1||
      time_frame==PERIOD_W1)
      if(cur_bar_time_struct.day!=prev_bar_time_struct.day)
        {
         prev_bar_time_dig[0]=cur_bar_time_dig[0];
         TimeToStruct(prev_bar_time_dig[0],prev_bar_time_struct);
         return(true);
        };
   if(time_frame==PERIOD_MN1)
      if(cur_bar_time_struct.mon!=prev_bar_time_struct.mon)
        {
         prev_bar_time_dig[0]=cur_bar_time_dig[0];
         TimeToStruct(prev_bar_time_dig[0],prev_bar_time_struct);
         return(true);
        };
   return(false);
  };
//+------------------------------------------------------------------+
